/// <reference types="node" />
import { Agile, StorageKey, StateObserver, StatePersistent, Observer, PersistentKey, StateIngestConfigInterface } from '../internal';
export declare class State<ValueType = any> {
    agileInstance: () => Agile;
    _key?: StateKey;
    valueType?: string;
    isSet: boolean;
    isPlaceholder: boolean;
    initialStateValue: ValueType;
    _value: ValueType;
    previousStateValue: ValueType;
    nextStateValue: ValueType;
    observers: StateObserversInterface<ValueType>;
    sideEffects: {
        [key: string]: SideEffectInterface<State<ValueType>>;
    };
    computeValueMethod?: ComputeValueMethod<ValueType>;
    computeExistsMethod: ComputeExistsMethod<ValueType>;
    isPersisted: boolean;
    persistent: StatePersistent | undefined;
    watchers: {
        [key: string]: StateWatcherCallback<ValueType>;
    };
    currentInterval?: NodeJS.Timer | number;
    /**
     * A State manages a piece of Information
     * that we need to remember globally at a later point in time.
     * While providing a toolkit to use and mutate this piece of Information.
     *
     * You can create as many global States as you need.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/)
     *
     * @public
     * @param agileInstance - Instance of Agile the State belongs to.
     * @param initialValue - Initial value of the State.
     * @param config - Configuration object
     */
    constructor(agileInstance: Agile, initialValue: ValueType, config?: StateConfigInterface);
    /**
     * Assigns a new value to the State
     * and rerenders all subscribed Components.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/properties#value)
     *
     * @public
     * @param value - New State value.
     */
    set value(value: ValueType);
    /**
     * Returns a reference-free version of the current State value.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/properties#value)
     *
     * @public
     */
    get value(): ValueType;
    /**
     * Updates the key/name identifier of the State.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/properties#key)
     *
     * @public
     * @param value - New key/name identifier.
     */
    set key(value: StateKey | undefined);
    /**
     * Returns the key/name identifier of the State.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/properties#key)
     *
     * @public
     */
    get key(): StateKey | undefined;
    /**
     * Updates the key/name identifier of the State.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#setkey)
     *
     * @public
     * @param value - New key/name identifier.
     */
    setKey(value: StateKey | undefined): this;
    /**
     * Assigns a new value to the State
     * and re-renders all subscribed UI-Components.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#set)
     *
     * @public
     * @param value - New State value
     * @param config - Configuration object
     */
    set(value: ValueType | ((value: ValueType) => ValueType), config?: StateIngestConfigInterface): this;
    /**
     * Ingests the State without any specified new value into the runtime.
     *
     * Since no new value was defined either the new State value is computed
     * based on a compute method (Computed Class)
     * or the `nextStateValue` is taken as the next State value.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#ingest)
     *
     * @internal
     * @param config - Configuration object
     */
    ingest(config?: StateIngestConfigInterface): this;
    /**
     * Assigns a primitive type to the State
     * which constrains the State value on the specified type
     * to ensure basic typesafety in Javascript.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#type)
     *
     * @public
     * @param type - Primitive type the State value must follow (`String`, `Boolean`, `Array`, `Object`, `Number`).
     */
    type(type: any): this;
    /**
     * Undoes the latest State value change.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#undo)
     *
     * @public
     * @param config - Configuration object
     */
    undo(config?: StateIngestConfigInterface): this;
    /**
     * Resets the State value to its initial value.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#reset)
     *
     * @public
     * @param config - Configuration object
     */
    reset(config?: StateIngestConfigInterface): this;
    /**
     * Merges the specified `targetWithChanges` object into the current State value.
     * This merge can differ for different value combinations:
     * - If the current State value is an `object`, it does a partial update for the object.
     * - If the current State value is an `array` and the specified argument is an array too,
     *   it concatenates the current State value with the value of the argument.
     * - If the current State value is neither an `object` nor an `array`, the patch can't be performed.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#patch)
     *
     * @public
     * @param targetWithChanges - Object to be merged into the current State value.
     * @param config - Configuration object
     */
    patch(targetWithChanges: Object, config?: PatchConfigInterface): this;
    /**
     * Fires on each State value change.
     *
     * Returns the key/name identifier of the created watcher callback.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#watch)
     *
     * @public
     * @param callback - A function to be executed on each State value change.
     */
    watch(callback: StateWatcherCallback<ValueType>): string;
    /**
     * Fires on each State value change.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#watch)
     *
     * @public
     * @param key - Key/Name identifier of the watcher callback.
     * @param callback - A function to be executed on each State value change.
     */
    watch(key: string, callback: StateWatcherCallback<ValueType>): this;
    /**
     * Removes a watcher callback with the specified key/name identifier from the State.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#removewatcher)
     *
     * @public
     * @param key - Key/Name identifier of the watcher callback to be removed.
     */
    removeWatcher(key: string): this;
    /**
     * Returns a boolean indicating whether a watcher callback with the specified `key`
     * exists in the State or not.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#haswatcher)
     *
     * @public
     * @param key - Key/Name identifier of the watcher callback to be checked for existence.
     */
    hasWatcher(key: string): boolean;
    /**
     * Fires on the initial State value assignment and then destroys itself.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#oninaugurated)
     *
     * @public
     * @param callback - A function to be executed after the first State value assignment.
     */
    onInaugurated(callback: StateWatcherCallback<ValueType>): this;
    /**
     * Preserves the State `value` in the corresponding external Storage.
     *
     * The State key/name is used as the unique identifier for the Persistent.
     * If that is not desired or the State has no unique identifier,
     * please specify a separate unique identifier for the Persistent.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#persist)
     *
     * @public
     * @param config - Configuration object
     */
    persist(config?: StatePersistentConfigInterface): this;
    /**
     * Preserves the State `value` in the corresponding external Storage.
     *
     * The specified key is used as the unique identifier for the Persistent.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#persist)
     *
     * @public
     * @param key - Key/Name identifier of Persistent.
     * @param config - Configuration object
     */
    persist(key?: PersistentKey, config?: StatePersistentConfigInterface): this;
    /**
     * Fires immediately after the persisted `value`
     * is loaded into the State from a corresponding external Storage.
     *
     * Registering such callback function makes only sense
     * when the State is [persisted](https://agile-ts.org/docs/core/state/methods/#persist).
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#onload)
     *
     * @public
     * @param callback - A function to be executed after the externally persisted `value` was loaded into the State.
     */
    onLoad(callback: (success: boolean) => void): this;
    /**
     * Repeatedly calls the specified callback function,
     * with a fixed time delay between each call.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#interval)
     *
     * @public
     * @param handler - A function to be executed every delay milliseconds.
     * @param delay - The time, in milliseconds (thousandths of a second),
     * the timer should delay in between executions of the specified function.
     */
    interval(handler: (value: ValueType) => ValueType, delay?: number): this;
    /**
     * Cancels a active timed, repeating action
     * which was previously established by a call to `interval()`.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#clearinterval)
     *
     * @public
     */
    clearInterval(): void;
    /**
     * Returns a boolean indicating whether the State exists.
     *
     * It calculates the value based on the `computeExistsMethod()`
     * and whether the State is a placeholder.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#exists)
     *
     * @public
     */
    get exists(): boolean;
    /**
     * Defines the method used to compute the existence of the State.
     *
     * It is retrieved on each `exists()` method call
     * to determine whether the State exists or not.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#computeexists)
     *
     * @public
     * @param method - Method to compute the existence of the State.
     */
    computeExists(method: ComputeExistsMethod<ValueType>): this;
    /**
     * Defines the method used to compute the value of the State.
     *
     * It is retrieved on each State value change,
     * in order to compute the new State value
     * based on the specified compute method.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#computevalue)
     *
     * @public
     * @param method - Method to compute the value of the State.
     */
    computeValue(method: ComputeValueMethod<ValueType>): this;
    /**
     * Returns a boolean indicating whether the specified value is equal to the current State value.
     *
     * Equivalent to `===` with the difference that it looks at the value
     * and not on the reference in the case of objects.
     *
     * @public
     * @param value - Value to be compared with the current State value.
     */
    is(value: ValueType): boolean;
    /**
     * Returns a boolean indicating whether the specified value is not equal to the current State value.
     *
     * Equivalent to `!==` with the difference that it looks at the value
     * and not on the reference in the case of objects.
     *
     * @public
     * @param value - Value to be compared with the current State value.
     */
    isNot(value: ValueType): boolean;
    /**
     * Inverts the current State value.
     *
     * Some examples are:
     * - `'jeff'` -> `'ffej'`
     * - `true` -> `false`
     * - `[1, 2, 3]` -> `[3, 2, 1]`
     * - `10` -> `-10`
     *
     * @public
     */
    invert(): this;
    /**
     *
     * Registers a `callback` function that is executed in the `runtime`
     * as a side effect of State changes.
     *
     * For example, it is called when the State value changes from 'jeff' to 'hans'.
     *
     * A typical side effect of a State change
     * could be the updating of the external Storage value.
     *
     * @internal
     * @param key - Key/Name identifier of the to register side effect.
     * @param callback - Callback function to be fired on each State value change.
     * @param config - Configuration object
     */
    addSideEffect<Instance extends State<ValueType>>(key: string, callback: SideEffectFunctionType<Instance>, config?: AddSideEffectConfigInterface): this;
    /**
     * Removes a side effect callback with the specified key/name identifier from the State.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#removesideeffect)
     *
     * @internal
     * @param key - Key/Name identifier of the side effect callback to be removed.
     */
    removeSideEffect(key: string): this;
    /**
     * Returns a boolean indicating whether a side effect callback with the specified `key`
     * exists in the State or not.
     *
     * [Learn more..](https://agile-ts.org/docs/core/state/methods/#hassideeffect)
     *
     * @internal
     * @param key - Key/Name identifier of the side effect callback to be checked for existence.
     */
    hasSideEffect(key: string): boolean;
    /**
     * Returns a boolean indicating whether the passed value
     * is of the before defined State `valueType` or not.
     *
     * @internal
     * @param value - Value to be checked for the correct type.
     */
    hasCorrectType(value: any): boolean;
    /**
     * Returns the persistable value of the State.
     *
     * @internal
     */
    getPersistableValue(): any;
}
export declare type StateKey = string | number;
export interface StateObserversInterface<ValueType = any> {
    /**
     * Observer responsible for the value of the State.
     */
    value: StateObserver<ValueType>;
}
export interface StateConfigInterface {
    /**
     * Key/Name identifier of the State.
     * @default undefined
     */
    key?: StateKey;
    /**
     * Observers that depend on the State.
     * @default []
     */
    dependents?: Array<Observer>;
    /**
     * Whether the State should be a placeholder
     * and therefore should only exist in the background.
     * @default false
     */
    isPlaceholder?: boolean;
}
export interface PatchConfigInterface extends StateIngestConfigInterface, PatchOptionConfigInterface {
}
export interface PatchOptionConfigInterface {
    /**
     * Whether to add new properties to the object during the merge.
     * @default true
     */
    addNewProperties?: boolean;
}
export interface StatePersistentConfigInterface {
    /**
     * Whether the Persistent should automatically load
     * the persisted value into the State after its instantiation.
     * @default true
     */
    loadValue?: boolean;
    /**
     * Key/Name identifier of Storages
     * in which the State value should be or is persisted.
     * @default [`defaultStorageKey`]
     */
    storageKeys?: StorageKey[];
    /**
     * Key/Name identifier of the default Storage of the specified Storage keys.
     *
     * The State value is loaded from the default Storage by default
     * and is only loaded from the remaining Storages (`storageKeys`)
     * if the loading from the default Storage failed.
     *
     * @default first index of the specified Storage keys or the AgileTs default Storage key
     */
    defaultStorageKey?: StorageKey;
}
export declare type StateWatcherCallback<T = any> = (value: T, key: string) => void;
export declare type ComputeValueMethod<T = any> = (value: T) => T;
export declare type ComputeExistsMethod<T = any> = (value: T) => boolean;
export declare type SideEffectFunctionType<Instance extends State> = (instance: Instance, properties?: {
    [key: string]: any;
}) => void;
export interface SideEffectInterface<Instance extends State> {
    /**
     * Callback function to be called on every State value change.
     * @return () => {}
     */
    callback: SideEffectFunctionType<Instance>;
    /**
     * Weight of the side effect.
     * The weight determines the order of execution of the registered side effects.
     * The higher the weight, the earlier it is executed.
     */
    weight: number;
}
export interface AddSideEffectConfigInterface {
    /**
     * Weight of the side effect.
     * The weight determines the order of execution of the registered side effects.
     * The higher the weight, the earlier it is executed.
     */
    weight?: number;
}
